/*
    This is part of mxldump.
    B-record generation routines.
    Copyright (C) 2006 2007  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

#include <sys/poll.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include "hex.h"

/***************************************************************************/
/* define interni alla libreria */

#define TIMEOUT		2000	// tempo attesa max echo B-record

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static char	Brecord[32];	// B-record

/***************************************************************************/
/* funzioni interne alla libreria */

void WriteBrecord(unsigned long int Addr,unsigned long int Data,char *Str,
		unsigned char Mode);
  /* Dato un valore a 32bit e l'indirizzo di memoria in cui va inserito
     crea un B-record contenente i dati specificati. La stringa viene creata
     all'interno del buffer Str e viene terminata. */

void ReadBrecord(unsigned long int Addr,char *Str,unsigned char Mode);
  /* Dato l'indirizzo di memoria da cui si desidera leggere crea un
     B-record che esegue l'operazione. La stringa viene creata
     all'interno del buffer Str e viene terminata. */

void EntryBrecord(unsigned long int Addr,char *Str);
  /* Crea un B-record contenente il punto di avvio del programma.
     La stringa viene creata nel buffer Str e viene terminata. */

int RecordExchange(int SerialFD,char *Record,unsigned long int *Recv);
  /* Invia la stringa passata sulla seriale e riceve l'eco fino all'arrivo
     del char '/'. L'attesa e' limitata ad un timeout. In caso di successo
     la funzione ritorna 0 e in Recv restituisce l'ultima dword ricevuta. */

/***************************************************************************/
/* define pubblici */

/* Available modes */
#define BYTE_M		0x00	// byte mode
#define WORD_M		0x40	// word mode
#define DWORD_M		0xc0	// dword mode

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

int RemoteRead(int SerialFD,unsigned long int Addr,unsigned long int *Data,
		unsigned char Mode);
  /* Read 32bit of data from remote board at given address using specified
     Mode and SerialFileDescriptor. In case of success the function
     returns 0 and *Data contains received data. */

int RemoteWrite(int SerialFD,unsigned long int Addr,unsigned long int Data,
		unsigned char Mode);
  /* Write 32bit of data to remote board at given address using specified
     Mode and SerialFileDescriptor. In case of success the function
     returns 0. */

int RemoteExec(int SerialFD,unsigned long int Addr);
  /* Start remote code execution from given address.
     In case of success the function returns 0. */
     
/***************************************************************************/
/* funzioni */

void WriteBrecord(unsigned long int Addr,unsigned long int Data,char *Str,
		unsigned char Mode)
{
  /* Dato un valore a 32bit e l'indirizzo di memoria in cui va inserito
     crea un B-record contenente i dati specificati. La stringa viene creata
     all'interno del buffer Str e viene terminata. */


     
  char	*Join;				// puntatore al punto di giunta



  Join=LongToHex(Addr,Str);		// inserisce l'indirizzo
  Join=ByteToHex(Mode|4,Join);		// mode + numero di bytes
  Join=LongToHex(Data,Join);		// inserisce la dword
  return;				// termina
}



void ReadBrecord(unsigned long int Addr,char *Str,unsigned char Mode)
{
  /* Dato l'indirizzo di memoria da cui si desidera leggere crea un
     B-record che esegue l'operazione. La stringa viene creata
     all'interno del buffer Str e viene terminata. */
  
  
  
  char	*Join;				// puntatore al punto di giunta



  Join=LongToHex(Addr,Str);		// inserisce l'indirizzo
  Join=ByteToHex(Mode|0x20|4,Join);	// mode + read + numero di bytes
  return;				// termina
}



void EntryBrecord(unsigned long int Addr,char *Str)
{
  /* Crea un B-record contenente il punto di avvio del programma.
     La stringa viene creata nel buffer Str e viene terminata. */
  

  
  char	*Join;				// puntatore al punto di giunta
  

  
  Join=LongToHex(Addr,Str);		// inserisce l'indirizzo
  Join=ByteToHex(0,Join);		// inserisce il numero di bytes
  return;				// termina
}



int RecordExchange(int SerialFD,char *Record,unsigned long int *Recv)
{
  /* Invia la stringa passata sulla seriale e riceve l'eco fino all'arrivo
     del char '/'. L'attesa e' limitata ad un timeout. In caso di successo
     la funzione ritorna 0 e in Recv restituisce l'ultima dword ricevuta. */



  struct pollfd	SerInput;			// polling structure
  int		EchoLen;			// lunghezza dell'echo
  char		Echo[32];			// buffer ricezione echo



  /* imposta i valori per poll */
  SerInput.fd=SerialFD;				// fd da sorvegliare
  SerInput.events=POLLIN|POLLPRI;		// eventi interessanti
  
  /* invia il comando */
  //printf("--> %s\n",Record);			// stampa il record
  write(SerialFD,Record,strlen(Record));	// invia la stringa
  
  /* attende la conferma operazione */
  do
  {
    if(poll(&SerInput,1,TIMEOUT)==0)		// attende dati dalla seriale
    {
      /* no data from serial port */
      fprintf(stderr,"%s: no data received.\n",__FUNCTION__);
      						// visualizza l'errore
      return 1;					// termina con insuccesso
    }
    else
    {
      /* legge i dati dalla seriale */
      EchoLen=read(SerialFD,Echo,30);		// riceve l'echo
    }
  }while(Echo[EchoLen-1]!='/');			// attende il terminatore
  if(Recv!=NULL)
    *Recv=HexToLong(&Echo[EchoLen-9]);		// restituisce il long ricevuto
  return 0;					// termina con successo
}



int RemoteRead(int SerialFD,unsigned long int Addr,unsigned long int *Data,
		unsigned char Mode)
{
  /* Read 32bit of data from remote board at given address using specified
     Mode and SerialFileDescriptor. In case of success the function
     returns 0 and *Data contains received data. */
  
  
  
  char	*JoinPtr;				// puntatore al punto di giunta



  /* crea il B-record per l'operazione */
  JoinPtr=LongToHex(Addr,Brecord);		// inserisce l'indirizzo
  JoinPtr=ByteToHex(Mode|0x20|4,JoinPtr);	// mode + read + numero bytes

  /* invia il B-record */
  return(RecordExchange(SerialFD,Brecord,Data));
}



int RemoteWrite(int SerialFD,unsigned long int Addr,unsigned long int Data,
		unsigned char Mode)
{
  /* Write 32bit of data to remote board at given address using specified
     Mode and SerialFileDescriptor. In case of success the function
     returns 0. */
}



int RemoteExec(int SerialFD,unsigned long int Addr)
{
  /* Start remote code execution from given address.
     In case of success the function returns 0. */
}

/* $Source$ */

