/*
    This is part of mxldump.
    Hexadecimal conversion routines.
    Copyright (C) 2006 2007  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

/***************************************************************************/
/* define interni alla libreria */

#define UPPERCASE	7
#define LOWERCASE	39
#define LETTER		UPPERCASE		// lettere utilizzate

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

char HighChar(unsigned char Byte);
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble alto. */

char LowChar(unsigned char Byte);
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble basso. */

char *ByteToHex(unsigned char Byte,char *Str);
  /* Dato in ingresso un valore a 8 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa viene terminata,
     in uscita viene restituito un puntatore al terminatore. */
     
char *LongToHex(unsigned long int Long,char *Str);
  /* Dato in ingresso un valore a 32 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa viene terminata,
     in uscita viene restituito un puntatore al terminatore. */

unsigned long int HexToLong(char *Str);
  /* Data in ingresso una stringa rappresentante un valore a 32 bit in
     forma esadecimale restituisce il valore coma intero senza segno. */

/***************************************************************************/
/* funzioni */

char HighChar(unsigned char Byte)
{
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble alto. */
  
  
  
  unsigned char	OutValue;		// cifra esadecimale in uscita



  OutValue=(Byte>>4)|0x30;		// lo trasforma in cifra
  if(OutValue>0x39)
    OutValue+=LETTER;			// lo trasforma in lettera
  return((char)OutValue);		// termina
}



char LowChar(unsigned char Byte)
{
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble basso. */



  unsigned char	OutValue;		// cifra esadecimale in uscita



  OutValue=(Byte&0x0f)|0x30;		// lo trasforma in cifra
  if(OutValue>0x39)
    OutValue+=LETTER;			// lo trasforma in lettera
  return((char)OutValue);		// termina
}



char *ByteToHex(unsigned char Byte,char *Str)
{
  /* Dato in ingresso un valore a 8 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa viene terminata,
     in uscita viene restituito un puntatore al terminatore. */



  Str[0]=HighChar(Byte);		// inserisce la cifra alta
  Str[1]=LowChar(Byte);			// inserisce la cifra bassa
  Str[2]=0;				// termina la stringa
  return(&Str[2]);			// termina
}



char *LongToHex(unsigned long int Long,char *Str)
{
  /* Dato in ingresso un valore a 32 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa viene terminata,
     in uscita viene restituito un puntatore al terminatore. */
  
  
  
  Str[0]=HighChar((unsigned char)(Long>>24));	// primo byte
  Str[1]=LowChar((unsigned char)(Long>>24));
  Str[2]=HighChar((unsigned char)(Long>>16));	// secondo byte
  Str[3]=LowChar((unsigned char)(Long>>16));
  Str[4]=HighChar((unsigned char)(Long>>8));	// terzo byte
  Str[5]=LowChar((unsigned char)(Long>>8));
  Str[6]=HighChar((unsigned char)Long);		// quarto byte
  Str[7]=LowChar((unsigned char)Long);
  Str[8]=0;					// termina la stringa
  return(&Str[8]);				// termina
}



unsigned long int HexToLong(char *Str)
{
  /* Data in ingresso una stringa rappresentante un valore a 32 bit in
     forma esadecimale restituisce il valore coma intero senza segno. */


     
  int			i;
  unsigned long int	Value=0;



  for(i=0;i<8;++i)
  {
    if((Str[i]>=0x30)&&(Str[i]<=0x39))
    {
      Value<<=4;				// fa spazio per 4 bit
      Value|=(unsigned long int)(Str[i]&0x0f);	// aggiunge i nuovi 4 bit
    }
    else
    {
      if((Str[i]>='A')&&(Str[i]<='F'))
      {
	Value<<=4;				// fa spazio per 4 bit
	Value|=(unsigned long int)(Str[i]-55);	// aggiunge i nuovi 4 bit
      }
      else
      {
	if((Str[i]>='a')&&(Str[i]<='f'))
	{
	  Value<<=4;				// fa spazio per 4 bit
	  Value|=(unsigned long int)(Str[i]-87);
	  					// aggiunge i nuovi 4 bit
	}
	else
	{
	  return(Value);			// not an hex digit
	}
      }
    }
  }
  return(Value);				// termina
}

/* $Source$ */

