/*
    Fresscale Dragon Ball (C) memory management and debug utility.
    Copyright (C) 2006 2007  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h>
#include <string.h>
#include "hex.h"
#include "brec.h"
#include "serial.h"
#include "progbar.h"

/***************************************************************************/
/* define vari */

#define TIMEOUT		2000	// tempo attesa max
#define RAWLEN		40	// lunghezza massima riga

/* operation modes */
#define INVALID		0	// invalid calling name
#define MXLDUMP		1	// work as mxldump
#define MXLWRITE	2	// work as mxlwrite
#define MXLEXEC		3	// work as mxlexec
#define MXLRAW		4	// work as mxlraw
#define MXLSYNC		5	// work as mxlsync

/***************************************************************************/
/* costanti */

static const char Licenza[]=
	"mxldump version 0.05, Copyright (C) "
	"2006 2007  Alessandro Accattini\n";
static const char MxldumpHelp[]=
	"Usage: mxldump [OPTIONS]\n"
	"mxldump reads and dumps memory regions and working registers of a\n"
	"Dragon Ball (C) MC9328MXL microprocessor."
	"\n";
static const char MxlwriteHelp[]=
	"Usage: mxlwrite [OPTIONS] < datafile\n"
	"mxlwrite writes data file into memory of Dragon Ball (C) MC9328MXL\n"
	"microprocessor at given address."
	"\n";
static const char MxlexecHelp[]=
	"Usage: mxlexec [OPTIONS]\n"
	"mxlexec lets Dragon Ball (C) MC9328MXL execute code from given\n"
	"address."
	"\n";
static const char MxlrawHelp[]=
	"Usage: mxlraw [OPTIONS] < datafile\n"
	"mxlraw sends datafile to Dragon Ball (C) MC9328MXL as raw data\n"
	"B-records, without any translation."
	"\n";
static const char MxlsyncHelp[]=
	"Usage: mxlsync [OPTIONS]\n"
	"mxlsync syncronizes Dragon Ball (C) MC9328MXL firmware loader.\n"
	"\n";
static const char GeneralHelp[]=
	"\t-a ADDR\tspecify the memory address.\n"
	"\t-d DEV\tuse DEV as serial port.\n"
	"\t\t(default=%s)\n"
	"\t-h\tdisplay this help.\n"
	"\t-n N\tspecify how many dwords to read from memory.\n"
	"\t-s SPD\tselect the serial port speed:\n"
	"\t\t\t0  ->    300bps\t\t5  ->   9600bps\n"
	"\t\t\t1  ->    600bps\t\t6  ->  19200bps\n"
	"\t\t\t2  ->   1200bps\t\t7  ->  38400bps\n"
	"\t\t\t3  ->   2400bps\t\t8  ->  57600bps\n"
	"\t\t\t4  ->   4800bps\t\t9  -> 115200bps\n"
	"\t\t(default=%i)\n"
	"\t-B\tuse BYTE mode (8bit).\n"
	"\t-W\tuse WORD mode (16bit).\n"
	"\t-D\tuse DWORD mode (32bit) (default).\n"
	"\nReport bugs to bit.trapper@gmail.com\n\n";
static const char CallErr[]=
	"mxldump: unknown calling name, unknown action required.\n\n";

/***************************************************************************/
/* variabili */

int			SpeedTable[]={B300,B600,B1200,B2400,B4800,B9600,
					B19200,B38400,B57600,B115200};
int			Speed=7;		// velocita' impostata
int			Serial;			// serial port file descriptor
unsigned long int	Address=0;		// indirizzo di memoria
unsigned long int	DwordNum=1;		// numero DWORD da leggere
char			DataMode=DWORD_M;	// organizzazione dati
char			Buffer[4];		// buffer lettura file sorgente
char			Record[RAWLEN];		// B-record in uscita
char			HexAddr[10];		// forma esadecimale dell'addr.
char			HexData[10];		// forma esadecimale dei dati
unsigned long int	*Dword=(unsigned long int*)Buffer;	// dword ptr

/***************************************************************************/
/* prototipi funzioni */

int MxlDump(void);
  /* Legge dalla memoria del micro e visualizza sullo standard output. */

int MxlWrite(char RawMode);
  /* Legge dallo stdin e scrive nella memoria del micro. */

/***************************************************************************/
/* funzioni */

int MxlDump(void)
{
  /* Legge dalla memoria del micro e visualizza sullo standard output. */



  unsigned long int	Data[8];			// read data
  unsigned long int	RowAddress;			// first row address
  int			ReadVals;			// num read dwords
  int			i;



  if((DwordNum<=8)&&(DataMode=DWORD_M))
  {
    /* legge il valore di un registro */
    for(i=0;i<DwordNum;++i)
    {
      ReadBrecord(Address,Record,DataMode);		// crea il B-record
      if(RecordExchange(Serial,Record,&Data[0])!=0)	// invia il B-record
	return 1;					// termina con errore
      LongToHex(Address,HexAddr);			// address to hex
      LongToHex(Data[0],HexData);			// data to hex
      printf("[%s] = %s\n",HexAddr,HexData);		// visualizza il valore
      Address+=4;					// dword successiva
    }
  }
  else
  {
    /* legge piu' DWORD dalla memoria */
    while(DwordNum>0)
    {
      ReadVals=0;					// reset counter
      RowAddress=Address;				// save row address

      /* read up to 4 dwords */
      while((ReadVals<DwordNum)&&(ReadVals<4))
      {
        /* read 4 data bytes */
	ReadBrecord(Address,Record,DataMode);		// crea il B-record
	if(RecordExchange(Serial,Record,&Data[ReadVals])!=0)
							// invia il B-record
	  return 1;					// termina con errore
	Address+=4;					// next dword
	ReadVals++;					// update counter
      }

      /* print 1 data line (4 dwords) */
      LongToHex(RowAddress,HexAddr);			// address to hex
      printf("%s:  ",HexAddr);				// init row
      for(i=0;i<ReadVals;++i)
      {
	LongToHex(Data[i],HexData);			// data to hex
        printf("%s ",HexData);				// print row
      }
      printf("\n");					// end row
      DwordNum-=ReadVals;				// delete read dwords
    }
  }
  return 0;						// termina
}



int MxlWrite(char RawMode)
{
  /* Legge dallo stdin e scrive nella memoria del micro. */
  
  
  
  struct stat		FileInfo;		// file info structure
  int			SrcLen;			// numero bytes letti
  
  
  
  /* determina la dimensione del file proveniente dallo stdin */
  fstat(0,&FileInfo);				// getta la dimensione
  InitProgressBar(FileInfo.st_size);		// inizializza la barra

  /* processa il file proveniente dallo stdin */
  if(RawMode)
  {
    /* send file in RAW mode */
    while(fscanf(stdin,"%s",Record)!=EOF)	// legge una riga
    {
      IncrementBarOf(strlen(Record));		// aggiorna la barra
      if(RecordExchange(Serial,Record,NULL)!=0)	// invia la riga
	return 1;				// termina con errore
    }
    puts("\n");					// chiude la barra
  }
  else
  {
    /* send DATA file */
    do
    {
      SrcLen=fread(Buffer,1,4,stdin);			// legge 4 bytes
      WriteBrecord(Address,*Dword,Record,DataMode);	// crea il B-record
      Address+=4;					// aggiorna l'indirizzo
      IncrementBarOf(SrcLen);				// aggiorna la barra
      if(RecordExchange(Serial,Record,NULL)!=0)		// invia il B-record
	return 1;					// termina con errore
    }while(SrcLen==4);
    puts("\n");					// chiude la barra
  }
  return 0;					// termina
}





/***************************************************************************/

int main(int argc,char **argv)
{
  char			*Device="/dev/ttyS0";	// device utilizzato
  char			*VarVal;		// valore variabile d'ambiente
  int			Opzione;		// per la lettura delle opzioni
  int			OpReturn=0;		// return value
  char			OpMode=INVALID;		// operation mode



  /* individua il calling name */
  if(strcmp(argv[0],"mxldump")==0)
    OpMode=MXLDUMP;
  if(strcmp(argv[0],"mxlwrite")==0)
    OpMode=MXLWRITE;
  if(strcmp(argv[0],"mxlexec")==0)
    OpMode=MXLEXEC;
  if(strcmp(argv[0],"mxlraw")==0)
    OpMode=MXLRAW;
  if(strcmp(argv[0],"mxlsync")==0)
    OpMode=MXLSYNC;
  if(OpMode==INVALID)
  {
    fputs(CallErr,stderr);		// visualizza l'errore
    return 1;				// termina con insuccesso
  }

  /* legge le variabili d'ambiente */
  if((VarVal=getenv("MXLDUMP_DEV"))!=NULL)
    Device=VarVal;			// reimposta il device utilizzato
  if((VarVal=getenv("MXLDUMP_SPD"))!=NULL)
    sscanf(VarVal,"%i",&Speed);		// imposta la velocita'
  
  /* legge le opzioni presenti sulla riga di comando */
  while((Opzione=getopt(argc,argv,"ha:d:n:s:BWD"))!=-1)
  {
    switch(Opzione)
    {
      case 'h':
        switch(OpMode)
	{
	  case MXLDUMP:
	    puts(MxldumpHelp);		// visualizza l'help
	  break;
	  case MXLWRITE:
	    puts(MxlwriteHelp);		// visualizza l'help
	  break;
	  case MXLEXEC:
	    puts(MxlexecHelp);		// visualizza l'help
	  break;
	  case MXLRAW:
	    puts(MxlrawHelp);		// visualizza l'help
	  break;
	  case MXLSYNC:
	    puts(MxlsyncHelp);		// visualizza l'help
	  break;
	}
        printf(GeneralHelp,Device,Speed);	// completa l'help
	return 0;			// termina
      break;
      case 'd':
        Device=optarg;			// imposta il device utilizzato
      break;
      case 'a':
        sscanf(optarg,"%li",&Address);	// imposta l'indirizzo di memoria
      break;
      case 'n':
        sscanf(optarg,"%li",&DwordNum);	// imposta il numero di DWORD
      break;
      case 's':
        sscanf(optarg,"%i",&Speed);	// imposta la velocita'
      break;
      case 'B':
        DataMode=BYTE_M;		// BYTE mode
      break;
      case 'W':
        DataMode=WORD_M;		// WORD mode
      break;
      case 'D':
        DataMode=DWORD_M;		// DWORD mode
      break;
      default:
        puts("Unknown option, use -h to see help.");
	return 0;			// termina
      break;
    }
  }
  
  /* apre la porta seriale */
  //printf("Opening serial device: %s\n",Device);	// visualizza il device
  if((Serial=OpenSerialPort(Device,SpeedTable[Speed],1,0))<0)
  {
    perror("OpenSerialPort");			// visualizza l'errore
    return 1;					// termina
  }

  /* esegue la funzione richiesta */
  switch(OpMode)
  {
    case MXLDUMP:
      OpReturn=MxlDump();			// mxldump
    break;
    case MXLWRITE:
      OpReturn=MxlWrite(0);			// mxlwrite
    break;
    case MXLRAW:
      OpReturn=MxlWrite(1);			// mxlraw
    break;
    case MXLEXEC:
      EntryBrecord(Address,Record);		// crea l'entry point
      printf("sending B-record: %s\n",Record);	// visualizza il brecord
      RecordExchange(Serial,Record,NULL);	// invia il record
      //write(Serial,Record,strlen(Record));	// invia il record
      LongToHex(Address,HexAddr);		// converte l'indirizzo in hex
      printf("Started code at: %s\n",HexAddr);
      OpReturn=0;				// all OK
    break;
    case MXLSYNC:
      fputs("syncing loader... ",stdout);
      write(Serial,"a",1);			// invia il char di sincronismo
      sleep(1);					// aspetta 1sec
      read(Serial,Buffer,4);			// riceve il char di risposta
      if(Buffer[0]==':')			// testa la risposta
      {
	puts("OK");				// stampa il successo
	OpReturn=0;				// all OK
      }
      else
      {
	puts("failed.");			// stampa l'errore
	write(Serial,"\n",1);			// inizia un nuovo B-record
	OpReturn=1;				// some problems
      }
    break;
  }

  /* chide la porta seriale */
  CloseSerialPort(Serial);			// chiude la seriale
  return(OpReturn);				// termina
}

/* $Source$ */
  
