/*
    This is part of brecord.
    General serial port settings.
    Copyright (C) 2004 2006 2007  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

#define _POSIX_SOURCE	1	// POSIX compliant source

#include <sys/signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <termios.h>
#include <fcntl.h>
#include <unistd.h>

/***************************************************************************/
/* define interni alla libreria */

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static struct termios	OldSettings;	// impostazioni seriale
static struct termios	NewSettings;

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

int OpenSerialPort(char *DevName,int Speed,int BlockLen,int Timeout);
  /* Apre la porta seriale e imposta i parametri di comunicaione. In uscita
     restituisce il device handler della porta. In caso di fallimento
     restituisce -1 ed errno contiene l'errore. */

void CloseSerialPort(int DevFD);
  /* Reimposta i settaggi originali e chiude la porta seriale. */

/***************************************************************************/
/* funzioni */

int OpenSerialPort(char *DevName,int Speed,int BlockLen,int Timeout)
{
  /* Apre la porta seriale e imposta i parametri di comunicaione. In uscita
     restituisce il device handler della porta. In caso di fallimento
     restituisce -1 ed errno contiene l'errore. */


     
  int	SerialFD;			// serial file descriptor
  
  
  
  /* apre il device utilizzato */
  if((SerialFD=open(DevName,O_RDWR|O_NOCTTY))<0)
    return(SerialFD);			// termina con insuccesso
  
  /* imposta la seriale */
  tcgetattr(SerialFD,&OldSettings);		// salva le impostazioni
  NewSettings.c_cflag=Speed|CS8|CLOCAL|CREAD;	// velocita' + varie
  NewSettings.c_iflag=IGNPAR;			// ignora gli errori di parita'
  NewSettings.c_oflag=ONLCR;			// raw output
  NewSettings.c_lflag=0;			// no echo
  NewSettings.c_cc[VMIN]=BlockLen;		// numero char attesi
  NewSettings.c_cc[VTIME]=Timeout;		// timeout
  tcflush(SerialFD,TCIFLUSH);			// flusha il buffer
  tcsetattr(SerialFD,TCSANOW,&NewSettings);	// reimposta la seriale
  return(SerialFD);				// termina
}



void CloseSerialPort(int DevFD)
{
  /* Reimposta i settaggi originali e chiude la porta seriale. */
  
  
  
  tcflush(DevFD,TCIFLUSH);			// flusha il buffer
  tcsetattr(DevFD,TCSANOW,&OldSettings);	// reimposta la seriale
  close(DevFD);					// chiude la seriale
  return;					// termina
}

/* $Source$ */

